% CTA_demo1.m
% CTA_demo1.m demonstrates how to obtain Wavelet Multi-scale Descriptor (WMD) 
% from a single brain surface. 

clear;clc;close all;
dir = 'sample/';

% sample subject
subject = 'subject';

%% load data
h = 'left';
[surf, sphere, f] = CTA_load_surf(dir, subject, h);
figure;CTA_draw_surf(surf, f);
colormap('jet')

%% adjacency matrix and graph Laplacian
A = triangulation2adjacency(surf.faces, surf.vertices);
[D, L] = CTA_spectralgraph(A); 
    
%% heat kernel smoothing    
alpha = 1; % smoothing parameter
f_smooth = CTA_heat_smoothing(f, L, alpha);
figure;CTA_draw_surf(surf, f_smooth);
colormap('jet')

%% WMD parameters
m = 10; K = 80; J=5; designtype = 'abspline3'; 
lmax = sgwt_rough_lmax(L);
arange = [0 lmax];

[g,t]=sgwt_filter_design(lmax,J,'designtype',designtype,'lpfactor',K);
%sgwt_view_design(g,t,arange)

c = cell(numel(g),1);
for k=1:numel(g)
    c{k}=sgwt_cheby_coeff(g{k},m,m+1,arange);
end
    
%% computing WMD from cortical thickness    
WMD = sgwt_cheby_op(f,L,c,arange);    
figure;
set(gcf, 'Position', [100 100 1800, 400])
for i=1:J+1
    subplot(1,J+1,i);
    CTA_draw_surf(surf, WMD{i});
    colormap jet
end



