% CTA_demo3.m
% CTA_demo3.m demonstrates 
% 1) generation a smoothed brain surface from a single brain surface, 
% 2) downsampling using a template sphere
% 3) projecting a signal on a template sphere to a template surface
% 
% Note: Typically, Freesurfer data are not registered. In such cases, we
% project all signals from each brain surface to a template sphere by
% resampling, and then compute further statistics. Later, the result needs
% to be projected onto a brain surface to identify brain regions that show
% group differences. This demo shows a pipeline to resample a signal on a
% template sphere, and then projecting it back to a template brain. 

clear;
dir = 'sample/';

load FV7_oct.mat; % template sphere

% sample subject
subject = 'subject';

% load data
h = 'left';
[surf, sphere, f] = CTA_load_surf(dir, subject, h);
figure;CTA_draw_surf(surf, f);colormap jet;

%% re-sampling using a template sphere
f_FV = tri_values(sphere.vertices, f, FV);
figure;
subplot(1,2,1);CTA_draw_surf(FV, f_FV);title('Re-sampled signal');
subplot(1,2,2);CTA_draw_surf(sphere, f);title('Original signal');

%% genering a smoothed template brain surface
alph = 10;   % degree of smoothing
[template, index] = CTA_template(surf, sphere, alph, FV);
figure;CTA_draw_surf(template,f);


%% projecting a signal from the template sphere to the template brain surface

% If you compute a statistic (e.g, p-value) on a template sphere, 
% it later needs to a template brain surface. 

id = zeros(length(index),1);
for i=1:length(FV.vertices)
    temp = find(index == i);
    id(temp) = i;
end

figure;
subplot(1,2,1);CTA_draw_surf(template, f);title('Re-sampled signal');
subplot(1,2,2);CTA_draw_surf(template, f_FV(id));title('Projected (back) signal');
