function [theta,varphi]=EULERangles(surf);
%Given a spherical mesh, it generates Euler angles \theta and \varphi used
%in TMI 2007 publication. The TMI convention follows the most widely used
%mathematical convention and it is different from MATLAB convenstion.

n_vertex=size(surf.vertices,1);
c=mean(surf.vertices);  %mass center
surf.vertices=surf.vertices-kron(ones(n_vertex,1),c);  % translation

[theta,varphi,r] = cart2sph(surf.vertices(:,1),surf.vertices(:,2),surf.vertices(:,3));

% MATLAB coordinate systems are different from the convention used in the
% TMI paper.
temp = theta;
theta = pi/2 - varphi;
varphi = pi + temp;
