/* px_codensity.cc
 * ------------
 * Written by Patrick Perry
 * Original Version on July 5, 2004
 *
 * $Id: px_codensity.cc,v 1.1.1.1 2005/03/18 23:37:00 silva Exp $
 * $Log: px_codensity.cc,v $
 * Revision 1.1.1.1  2005/03/18 23:37:00  silva
 * Imported Sources
 *
 * Revision 1.1.1.1  2004/09/25 22:04:09  patperry
 * Imported sources
 *
 * Revision 1.2  2004/09/15 21:47:34  silva
 * *** empty log message ***
 *
 * Revision 1.1  2004/09/15 17:36:04  silva
 * Added px_ suffix for Plex 2.0 release.
 *
 * Revision 1.2  2004/07/28 23:23:29  patperry
 * Added subset and indices.
 *
 * Revision 1.1  2004/07/07 22:14:53  patperry
 * Added codensity function.
 *
 */

#include <iterator>
#include <utility>
#include <vector>
#include "PackedArrayIterator.h"
#include "PointSet.h"

extern "C" {
#include "mex.h"
#include "matrix.h"
}


extern "C" {
void
mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
  std::vector<int> kValues;
  double errorBound = 0.0;
  int kMax = 0;

  if ((nrhs != 3) && (nrhs != 4)) {
    mexErrMsgTxt("Three or four input arguments required.");
  } else if (nlhs > 2) {
    mexErrMsgTxt("Too many output arguments.");
  } else if (!mxIsDouble(prhs[0]) || mxIsComplex(prhs[0])
		   || mxIsSparse(prhs[0])) {
    mexErrMsgTxt("First argument must be a real, full, double matrix.");
  } else if (!mxIsDouble(prhs[1]) || mxIsComplex(prhs[1])
		   || mxIsSparse(prhs[1])) {
    mexErrMsgTxt("Second argument must be a real, full, double matrix.");
  } else if (!mxIsDouble(prhs[2]) || mxIsComplex(prhs[2])
		   || mxIsSparse(prhs[2])) {
    mexErrMsgTxt("Third argument must be a real, full, double matrix.");
  } else if (nrhs == 4) {
    if ((mxGetNumberOfElements(prhs[3]) != 1) || !mxIsDouble(prhs[3])
	   || mxIsComplex(prhs[3]) || mxIsSparse(prhs[3])) {
	 mexErrMsgTxt("Fourth argument must be a scalar double.");
    } else {
	 errorBound = std::max(0.0, mxGetScalar(prhs[2]));
    }
  } else if (mxGetM(prhs[0]) != mxGetM(prhs[1])) {
    mexErrMsgTxt("First and second arguments must have same number of rows.");
  }
  
  double * kData = mxGetPr(prhs[2]);
  int numKValues = mxGetNumberOfElements(prhs[2]);

  for (int i = 0; i < numKValues; ++i) {
    int k = static_cast<int>(kData[i]);
    
    if (k <= 0) {
	 mexErrMsgTxt("Third argument must contain positive integers.");
    }

    kValues.push_back(k);

    if (k > kMax) {
	 kMax = k;
    }
  }

  int dimension = mxGetM(prhs[0]);
  int numPoints = mxGetN(prhs[0]);
  double * pointData = mxGetPr(prhs[0]);
  PackedArrayIterator<double> pointsBegin(dimension, pointData);
  PackedArrayIterator<double> pointsEnd(dimension, 
								pointData + numPoints * dimension);
  PointSet<> pointSet(dimension, pointsBegin, pointsEnd);

  int numQueryPoints = mxGetN(prhs[1]);
  double * queryPointData = mxGetPr(prhs[1]);

  plhs[0] = mxCreateDoubleMatrix(numKValues, numQueryPoints, mxREAL);
  double * densityData = mxGetPr(plhs[0]);
  double * indexData = NULL;

  if (nlhs == 2) {
    plhs[1] = mxCreateDoubleMatrix(numKValues, numQueryPoints, mxREAL);
    indexData = mxGetPr(plhs[1]);
  }

  for (int i = 0; i < numQueryPoints; ++i) {
    std::vector<PointSet<>::NeighborQueryResult> neighbors;
    std::back_insert_iterator<std::vector<PointSet<>::NeighborQueryResult> >
	 ii(neighbors);

    pointSet.getNearestNeighbors(queryPointData + i * dimension, kMax, 
						   errorBound, ii);

    std::vector<int>::const_iterator it = kValues.begin();
    std::vector<int>::const_iterator end = kValues.end();
    
    while (it != end) {
	 int k = *it++;

	 *densityData++ = neighbors[k - 1].second;

	 if (indexData) {
	   *indexData++ = (neighbors[k - 1].first - pointData) / dimension;
	 }
    }
  }
}

}
