function [D, I] = px_codensity (X, A, K, epsilon)

%PX_CODENSITY -- calculate k-th nearest neighbor distances
%
% D      = PX_CODENSITY(X, A, K, epsilon)
% [D, I] = PX_CODENSITY(X, A, K, epsilon)
% 
% For each point in a set A, what is the Euclidean distance to the K-th
% nearest neighbour among the points in a different set X? This can be
% regarded as an inverse measure of the density of the set X near each
% point in A. PX_CODENSITY computes this exactly or approximately as
% desired.
%
% Inputs:
%
%   X : d-by-N matrix giving the coordinates of N points in R^d.
%
%   A : d-by-M matrix giving the coordinates of M points in R^d.
%
%   K : vector of non-negative integers of length nk.
%
%   epsilon : non-negative scalar
%
% Outputs:
%
%   D : nk-by-M matrix, such that D(i,j) is the distance between A(:,j)
%       and its K(i)-th nearest neighbour among the points X(:,k).
%
%   I : nk-by-M matrix, such that X(:,I(i,j)) is the K(i)-th nearest
%       neighbour of A(:,j) among the points X(:,k).
%
% [D, I] = PX_CODENSITY(X, A, K) solves the nearest-neighbor distance query
% exactly.
%
% [D, I] = PX_CODENSITY(X, A, K, epsilon) finds approximate nearest
% neighbors.  That is, the k-th distance is guaranteed to be no
% more than (1 + epsilon) times the actual distance.
%
% This function is implemented using  David Mount and Sunil Arya's
% Approximate Nearest Neighbor (ANN) library version 0.2.  More
% information about ANN can be found at http://www.cs.umd.edu/~mount/ANN/
%
%Plex Metric Data Toolbox version 2.5 by Vin de Silva, Patrick Perry and
%contributors. See PX_PLEXINFO for credits and licensing information.
%Released with Plex version 2.5. [2006-Jul-14]


% $Id: px_codensity.m,v 1.1.1.1 2005/03/18 23:37:00 silva Exp $
% $Log: px_codensity.m,v $
% Revision 1.1.1.1  2005/03/18 23:37:00  silva
% Imported Sources
%
% Revision 1.2  2004/10/22 01:28:13  silva
%
% Improving help-text and adding tags.
%
% Revision 1.1.1.1  2004/09/25 22:04:09  patperry
% Imported sources
%
% Revision 1.2  2004/09/16 00:29:56  silva
% Tidying up the help text.
%
% Revision 1.1  2004/09/15 17:36:04  silva
% Added px_ suffix for Plex 2.0 release.
%
% Revision 1.2  2004/07/28 23:23:29  patperry
% Added subset and indices.
%
% Revision 1.1  2004/07/07 22:14:53  patperry
% Added codensity function.
%
